(defun SelectionSort(L j)
  "
;; Examples:
 (selectionsort (iseq 10) 9)
 (selectionsort (sample (iseq 10) 10 t) 9)
"
  (if (= j 0)
      (print L)
    (let (
	  (max (elt L j))
	  (i 0)
	  maxi
	  )
      (while (< i j)
	(if (> (elt L i) max)
	    (setf
	     max (elt L i)
	     maxi i
	     )
	  )
	(incf i)
	)
      (if (> max (elt L j))
	  (setf
	   (elt L maxi) (elt L j)
	   (elt L j) max
	   )
	)
      (selectionsort L (- j 1))
      )
    )
  )

(defun BinarySearch(L i j x)
  "
;; Examples:
 (BinarySearch (iseq 10) 0 9 2)
 (BinarySearch (iseq 10) 0 9 12)
 (BinarySearch (iseq 10) 0 9 9)
 (BinarySearch (iseq 10) 0 9 -1)

 (BinarySearch (list 3 7 8 10 14 18 22 34) 0 7 25)
 (BinarySearch (list 3 7 8 10 14 18 22 34) 0 7 8)

 (time (BinarySearch (iseq 1000) 0 999 999))
 (time (BinarySearch (iseq 10000) 0 9999 9999))
 (time (BinarySearch (iseq 100000) 0 99999 99999))
 (time (BinarySearch (iseq 1000000) 0 999999 999999))
"
  (if (> i j)
      (print "Not found")
    (let ((k (+ i (floor (/ (- j i) 2)))))
      (print (elt L k))
      (if (= x (elt L k))
	  (print "Found")
	(if (< x (elt L k))
	    (BinarySearch L i (- k 1) x)
	  (BinarySearch L (+ k 1) j x)
	  )
	)
      )
    )
  )

(defun SequentialSearch(L x)
  "
;; Examples:
 (sequentialsearch (iseq 10) 2)
 (sequentialsearch (iseq 10) 12)
 (time (sequentialsearch (iseq 100000) 99999))
"
  (let (
	(n (length L))
	(i 0)
	)
    (while
	(and
	 (< i (- n 1))
	 (/= (elt L i) x)
	 )
      (incf i)
      )
    (if (= (elt L i) x)
	(print "Found")
      (print "Not found")
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;     Function Mergesort - relies on function merge, and recursion:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mergesort(x)
  "
;; Examples:
 (mergesort-tester 10)
 (mergesort-tester 20)
 (mergesort-tester 30)
"
  (let ((n (length x)))
    ;; if the list is of length 1, simply return it (no comparisons necessary)
    (if (= 1 n) x
      ;; otherwise,
      (let* (
	     ;; split the list -
	     (half (floor (/ n 2)))
	     (lhalf (select x (iseq half)))
	     (rhalf (select x (iseq half (- n 1))))
	     )
	;; and merge the two halves once they've been sorted:
	(merge
	 (mergesort lhalf)
	 (mergesort rhalf)
	 )
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;    Function merge - merges two lists (recursive)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun merge(x y)
  ;; if either list is of length 0, then we can simply return the other half
  (if (or
       (= 0 (length x))
       (= 0 (length y))
       )
      ;; if x is empty, return y; else return x:
      (if (= 0 (length x)) y x)
    ;; otherwise, 
    (let ()
      ;; increment the (global) variable compares,
      (incf compares)
      ;; and compare the first elements: 
      (if (< (first x) (first y))
	  ;; we plop (first x) into the top spot, merge the rest, and combine
	  ;; the results into a single list:
	  (combine (list (first x) (merge (rest x) y)))
	;; otherwise, we plop (first y) into the top spot, merge the rest, and
	;; combine the results into a single list: 
	(combine (list (first y) (merge x (rest y))))
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   Function comparisons - simply calculates the theoretical "worst-case
;;   scenario"
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun comparisons(n) (- (* n (/ (ln n) (ln 2))) (- n 1)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   Function mergesort-tester - tests mergesort
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mergesort-tester(n)
  (def compares 0)
  (def a (sample (iseq (* 2 n)) n))
  (print a)
  (print (mergesort a))
  (print compares)
  (print (comparisons n))
  nil
  )

#|
Some results of testing mergesort:

;; Examples:
 (mergesort-tester 10)
 (mergesort-tester 20)
 (mergesort-tester 30)

(5 1 2 12 11 9 7 14 0 8) 
(0 1 2 5 7 8 9 11 12 14) 
23 
24.219280948873624 
nil 
(20 4 33 3 27 28 9 32 0 6 22 39 17 23 13 26 36 14 25 11) 
(0 3 4 6 9 11 13 14 17 20 22 23 25 26 27 28 32 33 36 39) 
66 
67.43856189774725 
nil 
(46 12 22 19 38 8 25 42 15 29 59 28 57 49 5 58 44 20 3 18 31 51 14 21 1 41 33 36 24 23) 
(1 3 5 8 12 14 15 18 19 20 21 22 23 24 25 28 29 31 33 36 38 41 42 44 46 49 51 57 58 59) 
113 
118.20671786825557 
nil 
|#
